/* ldapuser.c */
/*
 * Copyright (C) 1998/8 Takuto Okuno.
 * Nagoya Institute of technorogy in Japan.
 *
 * Get user functions by ldap or local hard disk.
 */
#include <pwd.h>

#include "../sslapp.h"

#define SSL_CERT_USER "/etc/ssl.certusers"

int ok_pubkey2buf(pkey,buf)
EVP_PKEY *pkey;
char *buf;
{
    int i,max;
    unsigned char *num=NULL,bin[2048];
#ifndef NO_RSA
    if (pkey->type == EVP_PKEY_RSA){
        num=(unsigned char*)pkey->pkey.rsa->n;
        max=BN_bn2bin(num,&bin[1]);
        if (bin[1] & 0x80)
            max++;
        else    buf++;

        sprintf(buf,"%02x",bin[1]);
        for(i=2;i<max;i++)
            sprintf(buf,"%s:%02x",buf,bin[i]);
    }else
#endif
#ifndef NO_DSA
    if (pkey->type == EVP_PKEY_DSA){
        if (pkey->pkey.dsa->pub_key != NULL)
	    num=(unsigned char*)pkey->pkey.dsa->pub_key;

        max=BN_bn2bin(num,&bin[1]);
        if (bin[1] & 0x80)
            max++;
        else    buf++;

        sprintf(buf,"%02x",bin[1]);
        for(i=2;i<max;i++)
            sprintf(buf,"%s:%02x",buf,bin[i]);

    }
#endif
}

/*--------------------------------------------------
    Local Hard Disk check user
--------------------------------------------------*/
int ok_local_get_user(name,x509)
char *name;
X509 *x509;
{
    FILE *fp;
    char *p,*prot,*sn,*ca;
    char issuer[256],line[2048];
    int  xserial,num;

    X509_NAME_oneline(X509_get_issuer_name(x509),issuer,sizeof(issuer));
    xserial=(int)ASN1_INTEGER_get(X509_get_serialNumber(x509));

    if(ssl_verbose_flag)
	fprintf(stderr,"get serial=%d:issuer=%s\n",xserial,issuer);
    
    if ((fp = fopen(SSL_CERT_USER,"r")) != NULL) {
	while (fgets(line, sizeof(line), fp) != NULL)
	    if ((p = strchr(line, (char)'\n')) != NULL) {
		*p = '\0';
                if (line[0] == '#')
                    continue;

		if((p=strchr(line,(char)':'))!=NULL){
		    *p='\0'; p++;

		    if((sn=strchr(p,(char)':'))==NULL)
			break;
		    *sn='\0'; sn++; prot=p;

		    while(1){ /* get protocol and check */
			if((p=strchr(prot,(char)','))!=NULL)
			    *p='\0';
			if(!strcmp(prot,"ftp"))
			    break;

			if(p==NULL){
			    fclose(fp); return(0);
			}
			prot=(++p);
		    }

		    if((p=strchr(sn,(char)':'))==NULL)
		      break;
		    *p='\0'; p++; ca=p;
		    num=atoi(sn);

		    if((num==xserial) && !strcmp(ca,issuer)){
			strcpy(name,line);
			return(1);
		    }
		}
	    }
	(void) fclose(fp);
    }
    return (0);
}

/*--------------------------------------------------
    ldap check user
--------------------------------------------------*/
struct passwd *ok_ldap_get_pwd(x509)
X509 *x509;
{
return(NULL);
}

/*--------------------------------------------------
  check use ldap login or /bin/login
--------------------------------------------------*/
int ok_check_use_ldap(name,x509)
char *name;
X509 *x509;
{
  struct passwd *pwd;

  if((pwd=ok_ldap_get_pwd(x509))!=NULL){
    pwd->pw_passwd=NULL;
    /* ok_ldap_login(&pwd); */
  }else if(ok_local_get_user(name,x509)){
    return(1);
  }
  return(0);
}
