/* crl_vfy.c */
/*
 * Copyright (C) 1998/8 Takuto Okuno.
 * Nagoya Institute of technorogy in Japan.
 *
 * internal_verify() in X509_verify_cert() is not CRL check.
 * So I make ok_internal_verify().
 */

#include "sslapp.h"


static int null_callback(ok,e)
int ok;
X509_STORE_CTX *e;
{
  return(ok);
}


int ok_internal_verify(ctx)
X509_STORE_CTX *ctx;
{
/* --oku */
        X509_OBJECT obj;
        int xserial,chk_srl;
/* --oku */
        int i,ok=0,n;
        X509 *xs,*xi;
        EVP_PKEY *pkey=NULL;
        int (*cb)();

        cb=ctx->ctx->verify_cb;
        if (cb == NULL) cb=null_callback;

        n=sk_num(ctx->chain);
        ctx->error_depth=n-1;
        n--;
        xi=(X509 *)sk_value(ctx->chain,n);
        if (X509_NAME_cmp(X509_get_subject_name(xi),
                X509_get_issuer_name(xi)) == 0)
                xs=xi;
        else
	  {
                if (n <= 0)
		  {
                        ctx->error=X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE;
                        ctx->current_cert=xi;
                        ok=cb(0,ctx);
                        goto end;
		      }
                else
		  {
                        n--;
                        ctx->error_depth=n;
                        xs=(X509 *)sk_value(ctx->chain,n);
		      }
	      }

/*      ctx->error=0;  not needed */
        while (n >= 0)
	  {
                ctx->error_depth=n;
                if (!xs->valid)
		  {
                        if ((pkey=X509_get_pubkey(xi)) == NULL)
			  {
                                ctx->error=X509_V_ERR_UNABLE_TO_DECODE_ISSUER_PUBLIC_KEY;
                                ctx->current_cert=xi;
                                ok=(*cb)(0,ctx);
                                if (!ok) goto end;
			      }
                        if (X509_verify(xs,pkey) <= 0)
			  {
                                ctx->error=X509_V_ERR_CERT_SIGNATURE_FAILURE;
                                ctx->current_cert=xs;
                                ok=(*cb)(0,ctx);
                                if (!ok) goto end;
			      }
#if 0  /* --oku */
                        pkey=NULL;
#endif /* --oku */
                        i=X509_cmp_current_time(X509_get_notBefore(xs));
                        if (i == 0)
			  {
                                ctx->error=X509_V_ERR_ERROR_IN_CERT_NOT_BEFORE_FIELD;
                                ctx->current_cert=xs;
                                ok=(*cb)(0,ctx);
                                if (!ok) goto end;
			      }
                        if (i > 0)
			  {
                                ctx->error=X509_V_ERR_CERT_NOT_YET_VALID;
                                ctx->current_cert=xs;
                                ok=(*cb)(0,ctx);
                                if (!ok) goto end;
			      }
                        xs->valid=1;
		      }
                i=X509_cmp_current_time(X509_get_notAfter(xs));
                if (i == 0)
		  {
                        ctx->error=X509_V_ERR_ERROR_IN_CERT_NOT_AFTER_FIELD;
                        ctx->current_cert=xs;
                        ok=(*cb)(0,ctx);
                        if (!ok) goto end;
		      }

                if (i < 0)
		  {
                        ctx->error=X509_V_ERR_CERT_HAS_EXPIRED;
                        ctx->current_cert=xs;
                        ok=(*cb)(0,ctx);
                        if (!ok) goto end;
		      }

                /* CRL CHECK */
/* --oku */
                {
                  X509_REVOKED *r;
                  STACK *sk;
                  X509_CRL *xcrl;
#ifdef OK_DEBUG
                  char buf[256];

                  X509_NAME_oneline(X509_get_subject_name(xi),buf,sizeof(buf));
                  xserial=(int)ASN1_INTEGER_get(X509_get_serialNumber(xi));
                  fprintf(stderr,"issure %d:%s:%d\n",n,buf,xserial);
                  X509_NAME_oneline(X509_get_subject_name(xs),buf,sizeof(buf));
                  xserial=(int)ASN1_INTEGER_get(X509_get_serialNumber(xs));
                  fprintf(stderr,"now %d:%s:%d\n",n,buf,xserial);
#endif /* OK_DEBUG */

                  xserial=(int)ASN1_INTEGER_get(X509_get_serialNumber(xs));
                  ok=X509_STORE_get_by_subject(ctx,X509_LU_CRL,X509_get_subject_name(xi),&obj);

                  if(ok!=1){
                    ctx->error=X509_V_ERR_UNABLE_TO_GET_CRL;
                    ctx->current_cert=xs;
                    ok=(*cb)(0,ctx);
		    ok=1; /* Cannot get CRL ,but it is no error. */
/*                    if(!ok) goto end; */

                  }else{
                    xcrl=obj.data.crl;

                    if (X509_CRL_verify(xcrl,pkey) <= 0){
                        ctx->error=X509_V_ERR_CRL_SIGNATURE_FAILURE;
                        ctx->current_cert=xs;
                        ok=(*cb)(0,ctx);
                        if (!ok) goto end;
		      }
                    pkey=NULL;

                    i=X509_cmp_current_time(xcrl->crl->lastUpdate);
                    if (i == 0){
                        ctx->error=X509_V_ERR_ERROR_IN_CRL_LAST_UPDATE_FIELD;
                        ctx->current_cert=xs;
                        ok=(*cb)(0,ctx);
                        if (!ok) goto end;
		      }
                    if (i > 0){
                        ctx->error=X509_V_ERR_CRL_NOT_YET_VALID;
                        ctx->current_cert=xs;
                        ok=(*cb)(0,ctx);
                        if (!ok) goto end;
                      }

                    i=X509_cmp_current_time(xcrl->crl->nextUpdate);
                    if (i == 0){
                        ctx->error=X509_V_ERR_ERROR_IN_CRL_NEXT_UPDATE_FIELD;
                        ctx->current_cert=xs;
                        ok=(*cb)(0,ctx);
                        if (!ok) goto end;
                      }

                    if (i < 0){
                        ctx->error=X509_V_ERR_CRL_HAS_EXPIRED;
                        ctx->current_cert=xs;
                        ok=(*cb)(0,ctx);
                        if (!ok) goto end;
                      }

                    /* X509 REVOKED */
                    sk=sk_dup(xcrl->crl->revoked); /* CRL_INFO's revoked */

                    while ((r=(X509_REVOKED *)sk_pop(sk)) != NULL){
                      chk_srl=(int)ASN1_INTEGER_get(r->serialNumber);

                      if(chk_srl==xserial){
                        ctx->error=X509_V_ERR_CERT_REVOKED;
                        ctx->current_cert=xs;
                        ok=(*cb)(0,ctx);
                        if (!ok){ sk_free(sk); goto end; }
                      }
                    }
                    sk_free(sk);
                  }
                }
/* --oku */

                /* The last error (if any) is still in the error value */
                ctx->current_cert=xs;
                ok=(*cb)(1,ctx);
                if (!ok) goto end;

                n--;
                if (n >= 0)
		  {
                        xi=xs;
                        xs=(X509 *)sk_value(ctx->chain,n);
		      }
	      }
        ok=1;
end:
        return(ok);
}
