/* read_p12.c */
/* this code include pkcs12.c --oku 1998/8 */
/* pkcs12.c */
/* Copyright (C) 1997-8 Dr S N Henson (shenson@bigfoot.com) 
 * All Rights Reserved.
 * Any software using this code must include the following message in its
 * startup code or documentation and in any advertising material:
 * "This Product includes cryptographic software written by Dr S N Henson
 *  (shenson@bigfoot.com)"
 */
#ifdef USE_PKCS12

#include "sslapp.h"
#include "pkcs12.h"

int p12_twopass=0;
char *ssl_p12_file=NULL;

extern BIO *bio_err;
extern int ok_ssl_ctx_use_p12(SSL_CTX *ctx,PKCS12 *p12,unsigned char *pass,int passlen);
extern int ok_ssl_ctx_use_bags(SSL_CTX *ctx,STACK *bags,unsigned char *pass,int passlen);

/*--------------------------------------------------
    SSL CTX use Certificate and Key by PKCS12
--------------------------------------------------*/
int ok_ssl_ctx_use_p12_file(ctx,fname,twopass)
SSL_CTX *ctx;
char *fname;
int *twopass;
{
    int ret=0;
    unsigned char pass[64], macpass[64];
    BIO *in=NULL;
    PKCS12 *p12 = NULL;
    
    PKCS12_lib_init();

    in = BIO_new(BIO_s_file());
    if (BIO_read_filename (in,fname) <= 0) {
	perror (fname);
	goto end;
    }

    if (!(p12 = d2i_PKCS12_bio (in, NULL))) {
	ERR_print_errors(bio_err);
	goto end;
    }

    if(EVP_read_pw_string (pass,sizeof(pass),"Enter Import Password:",0)) {
	BIO_printf (bio_err, "Can't read Password\n");
	goto end;
    }

    if(twopass){
        if(EVP_read_pw_string (macpass,sizeof(macpass),"Enter MAC Password:",0)){
            BIO_printf (bio_err, "Can't read Password\n");
            goto end;
        }
    }else
	strcpy(macpass,pass);

    if (!PKCS12_verify_mac (p12, macpass, -1)) {
	BIO_printf (bio_err, "MAC verify errror: invalid password?\n");
	goto end;
     }
    
    BIO_printf (bio_err, "MAC verified OK\n");

    if (!ok_ssl_ctx_use_p12(ctx,p12,pass,-1)) {
	BIO_printf(bio_err, "Error outputting keys and certificates\n");
	ERR_print_errors (bio_err);
	goto end;
    }

    PKCS12_free(p12);
    ret=1;

  end:
    BIO_free(in);
    return(ret);
}

/*--------------------------------------------------
    SSL CTX use Certificate and Key by PKCS12
--------------------------------------------------*/
int ok_ssl_ctx_use_p12(ctx,p12,pass,passlen)
SSL_CTX *ctx;
PKCS12 *p12;
unsigned char *pass;
int passlen;
{
    STACK *asafes, *bags;
    int i, bagnid;
    PKCS7 *p7;

    if (!( asafes = M_PKCS12_unpack_authsafes (p12))) return 0;

    for (i = 0; i < sk_num (asafes); i++) {
	p7 = (PKCS7 *) sk_value (asafes, i);
	bagnid = OBJ_obj2nid (p7->type);
	
	if (bagnid == NID_pkcs7_data) {
	    bags = M_PKCS12_unpack_p7data (p7);
	} else if (bagnid == NID_pkcs7_encrypted) {
	    bags = M_PKCS12_unpack_p7encdata (p7, pass, passlen);
	} else continue;
	if (!bags) return 0;

	if (!ok_ssl_ctx_use_bags(ctx,bags,pass,passlen)) {
	    sk_pop_free (bags, PKCS12_SAFEBAG_free);
	    return 0;
	}
	sk_pop_free (bags, PKCS12_SAFEBAG_free);
    }
    sk_pop_free (asafes, PKCS7_free);
    return 1;
}

int ok_ssl_ctx_use_bags(ctx,bags,pass,passlen)
SSL_CTX *ctx;
STACK *bags;
unsigned char *pass;
int passlen;
{
    int i;

    for(i=0; i<sk_num(bags);i++) {
	if(!ok_ssl_ctx_use_bag(ctx,(PKCS12_SAFEBAG *)sk_value (bags, i),pass, passlen))
	    return 0;
    }
    return 1;
}

int ok_ssl_ctx_use_bag(ctx,bag, pass, passlen)
SSL_CTX *ctx;
PKCS12_SAFEBAG *bag;
unsigned char *pass;
int passlen;
{
    EVP_PKEY *pkey;
    PKCS8_PRIV_KEY_INFO *p8;
    static X509 *x509=NULL;
    static flag=1;
    int ret=1;
    char buf[256];

    switch (M_PKCS12_bag_type(bag)){
      case NID_keyBag:
	if (!(pkey =PKCS82PKEY(bag->value.keybag))) return 0;
	ret=SSL_CTX_use_PrivateKey(ctx,pkey);

	EVP_PKEY_free(pkey);
	break;

      case NID_pkcs8ShroudedKeyBag:
/*	if(EVP_read_pw_string (buf,sizeof(buf),"Enter Pass Password:",0)) {
	    BIO_printf (bio_err, "Can't read Password\n");
	    return(0);
	}
*/
	if (!(p8 =M_PKCS12_decrypt_skey(bag, pass, passlen))) return 0;
	if (!(pkey =PKCS82PKEY(p8))) return 0;
        ret=SSL_CTX_use_PrivateKey(ctx,pkey);

	PKCS8_PRIV_KEY_INFO_free(p8);
	EVP_PKEY_free(pkey);
	break;

      case NID_certBag:
	if (M_PKCS12_cert_bag_type(bag) !=NID_x509Certificate) return 1;
	if (!(x509 =M_PKCS12_certbag2x509(bag))) return 0;
        if(flag){
	    ret=SSL_CTX_use_certificate(ctx,x509);
	    flag=0;}
	X509_free(x509);
	break;

      case NID_safeContentsBag:
	return ok_ssl_ctx_use_bags(ctx, bag->value.safes, pass,passlen);
					
      default:
	BIO_printf (bio_err, "Warning unsupported bag type: ");
	i2a_ASN1_OBJECT (bio_err, bag->type);
	BIO_printf (bio_err, "\n");
	return 1;
	break;
    }
    return ret;
}

#endif /* USE_PKCS12 */
