/* sslapp.c	- ssl application code */

/*
 * The modifications to support SSLeay were done by Tim Hudson
 * tjh@cryptsoft.com
 *
 * You can do whatever you like with these patches except pretend that
 * you wrote them.
 *
 * Email ssl-users-request@lists.cryptsoft.com to get instructions on how to
 * join the mailing list that discusses SSLeay and also these patches.
 *
 */

#ifdef USE_SSL

#include "sslapp.h"

SSL_CTX *ssl_ctx;
SSL *ssl_con;
int ssl_debug_flag=0;
int ssl_only_flag=0;
int ssl_active_flag=0;
int ssl_verify_flag=SSL_VERIFY_NONE;
int ssl_secure_flag=0;
int ssl_certsok_flag=0;
int ssl_cert_required=0;
int ssl_verbose_flag=0;
int ssl_disabled_flag=0;

#ifdef USE_SIGN_LOGIN /* --oku */
int ssl_sign_only_flag=0;
int ssl_digital_id_flag=0;
X509 *peer_x509=NULL;

int ok_internal_verify(X509_STORE_CTX *ctx); /* SSLeay v0.9.0 doesnot CRL check. So I make it. */
#endif /* --oku */
char *ssl_cert_file=NULL;
char *ssl_key_file=NULL;
char *ssl_cipher_list=NULL;
char *ssl_log_file=NULL;
char *ssl_ca_file=NULL;
char *ssl_ca_path=NULL;

#ifdef USE_PKCS12
extern int p12_twopass;
extern char *ssl_p12_file;
#endif /* USE_PKCS12 */



/*--------------------------------------------------
    SSL Verify callback
--------------------------------------------------*/
#ifdef SSLEAY8
int verify_callback(ok,ctx)
int ok;
X509_STORE_CTX *ctx;
#else /* !SSLEAY8 */
int verify_callback(ok, xs, xi, depth, error)
int ok;
char *xs, *xi;
int depth, error;
#endif /* SSLEAY8 */
{
#ifdef SSLEAY8
    int depth=ctx->error_depth;
    int error=ctx->error;
    char *xs,*s,buf[256];

/*    xs=(char *)X509_STORE_CTX_get_current_cert(ctx);*/
    s=X509_NAME_oneline(X509_get_subject_name(ctx->current_cert),buf,sizeof(buf));
#endif /* SSLEAY8 */
    /*
     * If the verification fails, then don't remember the name.  However,
     * if we don't require a certificate, then return success which will
     * still allow us to set up an encrypted session.
     */
    if (ok){
#ifdef SSLEAY8
      if(ssl_debug_flag) BIO_printf(bio_err,"depth=%d %s\n",depth,buf);
#endif

    }else{
#ifdef SSLEAY8
      if(ssl_debug_flag)
	BIO_printf(bio_err,"depth=%d error=%d %s\n",depth,error,buf);
#else
      if(ssl_debug_flag)	
	BIO_printf(bio_err,"depth=%d error=%d\n",depth,error);
#endif
    }

    if (ssl_debug_flag)
        BIO_printf(bio_err,"verify_callback: returning %d\n",ok);

    return(ok);
}

/* fwd decl */
/*--------------------------------------------------
    SSL handshake infomation CallBack.
--------------------------------------------------*/
void apps_ssl_info_callback(s,where,ret)
SSL *s;
int where;
int ret;
{
    char *str;
    int w;

/* 1998/8 get_SIGN_LOGIN --oku */
#ifdef USE_SIGN_LOGIN
    if((s->state == SSL3_ST_CW_CERT_VRFY_A) ||
       (s->state == SSL3_ST_CW_CERT_VRFY_B) ||
       (s->state == SSL3_ST_SR_CERT_VRFY_A) ||
       (s->state == SSL3_ST_SR_CERT_VRFY_B))
	ssl_digital_id_flag=1;

    if(!ssl_verbose_flag)
	return;
#endif /* USE_SIGN_LOGIN */
/* oku */

    w=where& ~SSL_ST_MASK;

    if (w & SSL_ST_CONNECT) str="SSL_connect";
    else if (w & SSL_ST_ACCEPT) str="SSL_accept";
    else str="undefined";

    if (where & SSL_CB_LOOP)
        BIO_printf(bio_err,"%s:%s\n",str,SSL_state_string_long(s));
    else if (where & SSL_CB_ALERT){
        str=(where & SSL_CB_READ)?"read":"write";
        BIO_printf(bio_err,"SSL3 alert %s:%s:%s\n",str,
                   SSL_alert_type_string_long(ret),
                   SSL_alert_desc_string_long(ret));
    }else if (where & SSL_CB_EXIT){
        if (ret == 0)
            BIO_printf(bio_err,"%s:failed in %s\n",
                       str,SSL_state_string_long(s));
        else if (ret < 0){
            BIO_printf(bio_err,"%s:error in %s\n",
                       str,SSL_state_string_long(s));
        }
    }
}

/*--------------------------------------------------
    BIO data dump CallBack
--------------------------------------------------*/
long bio_dump_cb(bio,cmd,argp,argi,argl,ret)
BIO *bio;
int cmd;
char *argp;
int argi;
long argl;
long ret;
{
    BIO *out;

    out=(BIO *)BIO_get_callback_arg(bio);
    if (out == NULL) return(ret);

    if (cmd == (BIO_CB_READ|BIO_CB_RETURN)){
        BIO_printf(out,"read from %08X [%08lX] (%d bytes => %ld (0x%X))\n",
                   bio,argp,argi,ret,ret);
        BIO_dump(out,argp,(int)ret);
        return(ret);
    }else if (cmd == (BIO_CB_WRITE|BIO_CB_RETURN)){
        BIO_printf(out,"write to %08X [%08lX] (%d bytes => %ld (0x%X))\n",
                   bio,argp,argi,ret,ret);
        BIO_dump(out,argp,(int)ret);
    }
    return(ret);
}

/*--------------------------------------------------
    DUMP CALLBACK Setting
--------------------------------------------------*/
int set_ssl_trace(SSL *con){
    if (con!=NULL) {
        if (ssl_debug_flag) {
            BIO_set_callback(SSL_get_rbio(con),bio_dump_cb);
            BIO_set_callback_arg(SSL_get_rbio(con),bio_err);
        }
    }
    return 0;
}

/*--------------------------------------------------
    SSLeay tool initialize
--------------------------------------------------*/
int do_ssleay_init(server)
int server;
{
  char *p;

  /* make sure we have somewhere we can log errors to */
  if (bio_err==NULL) {
    if ((bio_err=BIO_new(BIO_s_file()))!=NULL) {
      if (ssl_log_file==NULL)
	BIO_set_fp(bio_err,stderr,BIO_NOCLOSE);
      else {
	if (BIO_write_filename(bio_err,ssl_log_file)<=0) {
	  /* not a lot we can do */
	}
      }
    }
  }

  /* rather simple things these days ... the old SSL_LOG and SSL_ERR
   * vars are long gone now SSLeay8 has rolled around and we have 
   * a clean interface for doing things
   */
  if (ssl_debug_flag)
    BIO_printf(bio_err,"SSL_DEBUG_FLAG on\r\n");


  /* init things so we will get meaningful error messages
   * rather than numbers 
   */
  SSL_load_error_strings();

#ifdef SSLEAY8
  SSLeay_add_ssl_algorithms();
  ssl_ctx=(SSL_CTX *)SSL_CTX_new(SSLv23_method());

  /* we may require a temp 512 bit RSA key because of the
   * wonderful way export things work ... if so we generate
   * one now!
   */
  if (server) {
    if (SSL_CTX_need_tmp_RSA(ssl_ctx)) {
      RSA *rsa;

      if (ssl_debug_flag)
	  BIO_printf(bio_err,"Generating temp (512 bit) RSA key ...\r\n");
      rsa=RSA_generate_key(512,RSA_F4,NULL);
      if (ssl_debug_flag)
	  BIO_printf(bio_err,"Generation of temp (512 bit) RSA key done\r\n");
   
      if (!SSL_CTX_set_tmp_rsa(ssl_ctx,rsa)) {
	BIO_printf(bio_err,"Failed to assign generated temp RSA key!\r\n");
      }
      RSA_free(rsa);
      if (ssl_debug_flag)
	  BIO_printf(bio_err,"Assigned temp (512 bit) RSA key\r\n");
    }
  }

  /* also switch on all the interoperability and bug
   * workarounds so that we will communicate with people
   * that cannot read poorly written specs :-)
   */
  SSL_CTX_set_options(ssl_ctx,SSL_OP_ALL);
  SSL_CTX_set_quiet_shutdown(ssl_ctx,1);
  SSL_CTX_sess_set_cache_size(ssl_ctx,128);

#else /* !SSLEAY8 */
  ssl_ctx=(SSL_CTX *)SSL_CTX_new();
#endif /* SSLEAY8 */

  /* the user can set whatever ciphers they want to use */
  if (ssl_cipher_list==NULL) {
      p=getenv("SSL_CIPHER");
      if (p!=NULL)
        SSL_CTX_set_cipher_list(ssl_ctx,p);
  } else
      SSL_CTX_set_cipher_list(ssl_ctx,ssl_cipher_list);

  /* for verbose we use the 0.6.x info callback that I got
   * eric to finally add into the code :-) --tjh
   */
#ifndef USE_SIGN_LOGIN /* flag check is doing in callbacks */
  if (ssl_verbose_flag)
#endif
      SSL_CTX_set_info_callback(ssl_ctx,apps_ssl_info_callback);

  /* Add in any certificates if you want to here ... */
  if (ssl_cert_file) {
      if (!SSL_CTX_use_certificate_file(ssl_ctx, ssl_cert_file, 
		      X509_FILETYPE_PEM)) {
	  BIO_printf(bio_err,"Error loading %s: ",ssl_cert_file);
	  ERR_print_errors(bio_err);
	  BIO_printf(bio_err,"\r\n");
	  return(0);
      } else {
	  if (!ssl_key_file)
	      ssl_key_file = ssl_cert_file;
	  if (!SSL_CTX_use_RSAPrivateKey_file(ssl_ctx, ssl_key_file,
		      X509_FILETYPE_PEM)) {
	      BIO_printf(bio_err,"Error loading %s: ",ssl_key_file);
	      ERR_print_errors(bio_err);
	      BIO_printf(bio_err,"\r\n");
	      return(0);
	  }
      }
/* 1998/8 Add in any certificate by pkcs12 file --oku */
#ifdef USE_PKCS12
  }else if(ssl_p12_file){
      if(!ok_ssl_ctx_use_p12_file(ssl_ctx,ssl_p12_file,p12_twopass)){
	  BIO_printf(bio_err,"Error loading %s: ",ssl_p12_file);
	  ERR_print_errors(bio_err);
	  BIO_printf(bio_err,"\r\n");
	  return(0);
      }
#endif /* USE_PKCS12 */
  /* oku */
  }

  /* make sure we will find certificates in the standard
   * location ... otherwise we don't look anywhere for
   * these things which is going to make client certificate
   * exchange rather useless :-)
   */
#ifdef SSLEAY8
/** oku 1998/8 added **/
  SSL_CTX_set_verify(ssl_ctx,ssl_verify_flag,verify_callback);

#ifdef USE_SIGN_LOGIN /* SSLeay v0.9.0 doesn't CRL check. So I make it. */
  X509_STORE_set_verify_func(ssl_ctx->cert_store,ok_internal_verify);
#endif /* USE_SIGN_LOGIN */

  if(ssl_verify_flag)
      if((!SSL_CTX_load_verify_locations(ssl_ctx,ssl_ca_file,ssl_ca_path)) ||
	 (!SSL_CTX_set_default_verify_paths(ssl_ctx))){
	  BIO_printf(bio_err,"not set:SSL_load_verify_locations\n");
/*	  return(0);*/
      }
/** oku 1998/8 end **/
#else
  SSL_set_default_verify_paths(ssl_ctx);
#endif

  /* now create a connection */
  ssl_con=(SSL *)SSL_new(ssl_ctx);
#ifndef SSLEAY8
  SSL_set_verify(ssl_con,ssl_verify_flag,
		 ssl_verify_flag?verify_callback:NULL);
#endif
  return(1);
}

#else /* !USE_SSL */

/* something here to stop warnings if we build without SSL support */
static int dummy_func()
{
  int i;

  i++;
}

#endif /* USE_SSL */

